@@ -21,12 +21,21 @@ Done:
 
 	940710 AK Fixed support for 300Kbps 3.5" DD disks without 2m
 
+	20021102 AK: switched verify_later to be default on. Newer
+	kernels (2.4.x) cannot refrain from reading ahead, even if
+	BLKRA is set to 0, and this will cause failure when verifying
+	during formatting, because read-ahead will attempt to access
+	tracks that are not _yet_ formatted
+
 Todo:
 -	Allow reversing cylinder order, or perhaps have option to try as many
 	cylinders as happen to work (as in 2m).  Currently, if too many cylinders
 	are attempted it won't fail until the very end
  */
-
+#include <sys/types.h>
+#ifdef HAVE_SYS_SYSMACROS_H
+# include <sys/sysmacros.h>
+#endif
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
@@ -68,8 +77,9 @@ int index_size=146;
 char floppy_buffer[24 * 512];
 int verbosity = 3;
 static char noverify = 0;
+static char noformat = 0;
 static char dosverify = 0;
-static char verify_later = 0;
+static char verify_later = 1;
 short stretch;
 int cylinders, heads, sectors;
 int begin_cylinder, end_cylinder;
@@ -323,7 +333,8 @@ int format_track(struct params *fd, int cylinder, int head, int do_skew)
 	raw_cmd.length = nssect * sizeof(format_map_t);
 	raw_cmd.cmd_count = 6;
 	raw_cmd.cmd[0] = FD_FORMAT & ~fm_mode;
-	raw_cmd.cmd[1] = head << 2 | ( fd->drive & 3);
+	raw_cmd.cmd[1] = (head << 2 | ( fd->drive & 3)) ^
+	    (fd->swapSides ? 4 : 0);
 	raw_cmd.cmd[2] = fd->sizecode;
 	raw_cmd.cmd[3] = nssect;
 	raw_cmd.cmd[4] = fd->fmt_gap;
@@ -355,7 +366,8 @@ int format_track(struct params *fd, int cylinder, int head, int do_skew)
 		raw_cmd.data = floppy_buffer;
 		raw_cmd.cmd_count = 9;
 		raw_cmd.cmd[0] = FD_WRITE & ~fm_mode & ~0x80;
-		raw_cmd.cmd[1] = head << 2 | ( fd->drive & 3);
+		raw_cmd.cmd[1] = (head << 2 | ( fd->drive & 3)) ^
+		    (fd->swapSides ? 4 : 0);
 		raw_cmd.cmd[2] = cylinder;
 		raw_cmd.cmd[3] = head;
 		raw_cmd.cmd[4] = cur_sector;
@@ -490,6 +502,7 @@ int main(int argc, char **argv)
 
 	short retries;
 	short zeroBased=0;
+	short swapSides=0;
 	int n,rsize;
 	char *verify_buffer = NULL;
 	char dosdrive;
@@ -510,11 +523,15 @@ int main(int argc, char **argv)
 		(void *) &noverify,
 		"skip verification" },
 
+	{ '\0', "print-drive-deviation", 0, EO_TYPE_BYTE, 1, 0,
+		(void *) &noformat,
+		"print deviation, do not format " },
+
 	{ 'B', "dosverify", 0, EO_TYPE_BYTE, 1, 0,
 		(void *) &dosverify,
 		"verify disk using mbadblocks" },
 
-	{ 'V', "verify_later", 0, EO_TYPE_BYTE, 1, 0,
+	{ 'V', "verify_later", 1, EO_TYPE_BYTE, 1, 0,
 		(void *) &verify_later,
 		"verify floppy after all formatting is done" },
 
@@ -662,11 +679,6 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	fd[0].zeroBased = zeroBased;
-	if(zeroBased)
-		noverify = 1;
-
-
 	/* sanity checking */
 	if (sizecode < 0 || sizecode >= MAX_SIZECODE) {
 		fprintf(stderr,"Bad sizecode %d\n", sizecode);
@@ -779,7 +791,16 @@ int main(int argc, char **argv)
 				break;
 		}
 		stretch = geometry.stretch & 1;
-		
+#ifdef FD_ZEROBASED
+		if(geometry.stretch & FD_ZEROBASED) {
+			zeroBased = 1;
+		}
+#endif
+#ifdef FD_SWAPSIDES
+		if(geometry.stretch & FD_SWAPSIDES) {
+			swapSides = 1;
+		}
+#endif
 		mask |= SET_SECTORS | SET_CYLINDERS | 
 			SET_SIZECODE | SET_2M | SET_RATE;
 	} else {
@@ -831,6 +852,15 @@ int main(int argc, char **argv)
 				stretch = 0;
 		}
 	}
+
+	fd[0].zeroBased = zeroBased;
+#ifndef FD_ZEROBASED
+	if(zeroBased) {
+		noverify = 1;
+	}
+#endif
+	
+	fd[0].swapSides = swapSides;
 		
 	if (cylinders > fd[0].drvprm.tracks) {
 		fprintf(stderr,"too many cylinder for this drive\n");
@@ -900,8 +930,9 @@ int main(int argc, char **argv)
 			fprintf(stderr,
 				"In order to avoid this time consuming "
 				"measurement in the future,\n"
-				"add the following line to " DRIVEPRMFILE
-				":\ndrive%d: deviation=%d\n",
+				"add the following line to " DRIVEPRMFILE ":\n");
+			fprintf(stdout,
+				"drive%d: deviation=%d\n",
 				fd[0].drive, 
 				(fd[0].raw_capacity-old_capacity)*1000000/
 				old_capacity);
@@ -914,6 +945,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if(noformat)
+	    return 0;
+
 	/* FIXME.  Why is this needed? */
 	fd[0].raw_capacity -= 30;
 
@@ -971,7 +1005,11 @@ int main(int argc, char **argv)
 	parameters.head = heads;
 	parameters.track = cylinders;
 	parameters.size = cylinders * heads * sectors;
-	parameters.stretch = stretch;
+	parameters.stretch = stretch 
+#ifdef FD_ZEROBASED
+		| (zeroBased ? 4 : 0)
+#endif
+		| (swapSides ? 2 : 0);
 	parameters.gap = fd[0].gap;
 	if ( !use_2m)
 		fd0.rate = fd[0].rate;
@@ -1054,7 +1092,7 @@ int main(int argc, char **argv)
 	ioctl(fd[0].fd, FDFLUSH );
 	close(fd[0].fd);
 
-	if (! (mask & SET_DOSDRIVE ) && fd[0].drive < 2)
+	if (! (mask & SET_DOSDRIVE ) && fd[0].drive < 2 && !zeroBased)
 		dosdrive = fd[0].drive+'a';
 
 	if (dosdrive) {
@@ -1085,8 +1123,9 @@ int main(int argc, char **argv)
 			dosverify = 0;
 		}			
 	} else {
-		fprintf(stderr,
-			"\nwarning: mformat not called because DOS drive unknown\n");
+		if(!zeroBased)
+			fprintf(stderr,
+				"\nwarning: mformat not called because DOS drive unknown\n");
 		/*exit(1);*/
 		dosverify = 0;
 	}
